/*$Header: mustek_a2nu2.h,v0.1 2005.5.23 vinci Exp $*/
/* sane - Scanner Access Now Easy.
   Copyright (C) 2000 Mustek.
   Originally maintained by Mustek
   Copyright (C) 2001, 2002 by Henning Meier-Geinitz.

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.

   This file implements a SANE backend for Mustek 1200UB and similar 
   USB flatbed scanners.  */

#ifndef MUSTEK_USB2_H
#define MUSTEK_USB2_H

#include "../include/sane/sane.h"

#define ENABLE(OPTION)  s->opt[OPTION].cap &= ~SANE_CAP_INACTIVE
#define DISABLE(OPTION) s->opt[OPTION].cap |=  SANE_CAP_INACTIVE
#define IS_ACTIVE(OPTION) (((s->opt[OPTION].cap) & SANE_CAP_INACTIVE) == 0)
#define MM_PER_INCH	25.4

/*#define MUSTEK_USB_CONFIG_FILE "msam.ini"*/

#define SCAN_BUFFER_SIZE (64 * 1024)

#define MAX_RESOLUTIONS 12

/* const use in structures*/

/*scan mode*/
typedef WORD SCANMODE, *LPSCANMODE;
#define SM_TEXT		0x00
#define SM_GRAY		0x01
#define SM_RGB24	0x02
#define SM_GRAY10	0x03
#define SM_RGB30	0x04
#define SM_GRAY12	0x05
#define SM_RGB36	0x06
#define SM_GRAY14	0x07
#define SM_RGB42	0x08
#define SM_GRAY16	0x09
#define SM_RGB48	0x0a

/*pixel flavor*/
typedef BYTE PIXELFLAVOR, *LPPIXELFLAVOR;
#define PF_BlackIs0 0x00
#define PF_WhiteIs0 0x01

/*scan source*/
typedef BYTE SCANSOURCE, *LPSCANSOURCE;
#define SS_Reflective	0x00
#define SS_Positive		0x01
#define SS_Negative		0x02
#define SS_ADF			0x03

/*RGB order*/
typedef WORD RGBORDER, *LPRGBORDER;
#define RO_RGB 0x00
#define RO_BGR 0x01

/* structures use in parameters of export function*/

typedef struct tagGAMMAINFO
{
  SCANMODE smScanMode;
  WORD wInputGammaBits;
  WORD wOutputGammaBits;
} GAMMAINFO, *LPGAMMAINFO;

typedef struct tagGETPARAMETERS
{
  WORD wSourceXDPI;
  WORD wSourceYDPI;
  DWORD dwLineByteWidth;
  DWORD dwLength;
} GETPARAMETERS, *LPGETPARAMETERS;

typedef struct tagFRAME
{
  WORD x1;
  WORD y1;
  WORD x2;
  WORD y2;
} FRAME, *LPFRAME;

typedef struct tagSETPARAMETERS
{
  FRAME fmArea;
  WORD wTargetDPI;
  SCANMODE smScanMode;
  WORD wLinearThreshold;	/*threshold for Line art mode */
  PIXELFLAVOR pfPixelFlavor;
  SCANSOURCE ssScanSource;
  LPWORD pGammaTable;
} SETPARAMETERS, *LPSETPARAMETERS;

typedef struct tagIMAGEROWS
{
  RGBORDER roRgbOrder;
  WORD wWantedLineNum;
  WORD wXferedLineNum;
  LPBYTE pBuffer;
} IMAGEROWS, *LPIMAGEROWS;

/* export function in spicall module*/

BOOL _Open ();
BOOL _Close ();
BOOL GetDeviceStatus ();
BOOL PowerControl (BOOL isLampOn, BOOL isTaLampOn);
BOOL CarriageHome ();
BOOL GetGammaInfo (LPGAMMAINFO pGamaInfo);
BOOL SetParameters (LPSETPARAMETERS pSetParameters);
BOOL GetParameters (LPGETPARAMETERS pGetParameters);
BOOL StartScan ();
BOOL ReadScannedData (LPIMAGEROWS pImageRows);
BOOL StopScan ();
BOOL GetKeyStatus (BYTE * pKey);
BOOL IsTAConnected ();

enum Mustek_Usb_Option
{
  OPT_NUM_OPTS = 0,
  OPT_MODE_GROUP,
  OPT_MODE,
  OPT_SOURCE,
  OPT_RESOLUTION,
  OPT_PREVIEW,

  OPT_DEBUG_GROUP,
  OPT_AUTO_WARMUP,

  OPT_ENHANCEMENT_GROUP,
  OPT_THRESHOLD,
  OPT_GAMMA_VALUE,

  OPT_GEOMETRY_GROUP,
  OPT_TL_X,			/* top-left x */
  OPT_TL_Y,			/* top-left y */
  OPT_BR_X,			/* bottom-right x */
  OPT_BR_Y,			/* bottom-right y */
  /* must come last: */
  NUM_OPTIONS
};


typedef struct Scanner_Model
{
  /** @name Identification */
  /*@{ */

  /** A single lowercase word to be used in the configuration file. */
  SANE_String_Const name;

  /** Device vendor string. */
  SANE_String_Const vendor;

  /** Device model name. */
  SANE_String_Const model;

  /** Name of the firmware file. */
  SANE_String_Const firmware_name;

  /** @name Scanner model parameters */
  /*@{ */

  SANE_Int dpi_values[MAX_RESOLUTIONS];	/* possible resolutions */
  SANE_Fixed x_offset;		/* Start of scan area in mm */
  SANE_Fixed y_offset;		/* Start of scan area in mm */
  SANE_Fixed x_size;		/* Size of scan area in mm */
  SANE_Fixed y_size;		/* Size of scan area in mm */

  SANE_Fixed x_offset_ta;	/* Start of scan area in TA mode in mm */
  SANE_Fixed y_offset_ta;	/* Start of scan area in TA mode in mm */
  SANE_Fixed x_size_ta;		/* Size of scan area in TA mode in mm */
  SANE_Fixed y_size_ta;		/* Size of scan area in TA mode in mm */


  RGBORDER line_mode_color_order;	/* Order of the CCD/CIS colors */
  SANE_Fixed default_gamma_value;	/* Default gamma value */

  SANE_Bool is_cis;		/* Is this a CIS or CCD scanner? */

  SANE_Word flags;		/* Which hacks are needed for this scanner? */
  /*@} */
} Scanner_Model;

typedef struct Mustek_Scanner
{
  /* all the state needed to define a scan request: */
  struct Mustek_Scanner *next;

  SANE_Option_Descriptor opt[NUM_OPTIONS];
  Option_Value val[NUM_OPTIONS];
  WORD *gamma_table;
  SANE_Parameters params;   /**< SANE Parameters */
  Scanner_Model model;
}
Mustek_Scanner;

#endif
