/*Asic_motor.c: function of set motor parameter and action;*/
/* sane - Scanner Access Now Easy.

   Copyright (C) 2005 Mustek.
   Originally maintained by Mustek
   Author:Roy 2005.5.24

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.

   This file implements a SANE backend for Mustek 1200UB and similar 
   USB flatbed scanners.  */

#include "mustek_usb2_asic_motor.h"

STATUS LLFRamAccess(PAsic chip,LLF_RAMACCESS *RamAccess)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC,"LLFRamAccess:Enter\n");
	BYTE a[2];
	
	/*Set start address. Unit is a word.*/
	Mustek_SendData(chip, ES01_A0_HostStartAddr0_7, LOBYTE(RamAccess->LoStartAddress));
	
	if (RamAccess->IsOnChipGamma == ON_CHIP_FINAL_GAMMA)
	{/* final gamma */
		Mustek_SendData(chip, ES01_A1_HostStartAddr8_15, HIBYTE(RamAccess->LoStartAddress));
		Mustek_SendData(chip, ES01_A2_HostStartAddr16_21, LOBYTE(RamAccess->HiStartAddress) | ACCESS_GAMMA_RAM);
	}
	else if (RamAccess->IsOnChipGamma == ON_CHIP_PRE_GAMMA)
	{/* pre gamma */
		Mustek_SendData(chip, ES01_A1_HostStartAddr8_15, HIBYTE(RamAccess->LoStartAddress) | ES01_ACCESS_PRE_GAMMA);
		Mustek_SendData(chip, ES01_A2_HostStartAddr16_21, LOBYTE(RamAccess->HiStartAddress) | ACCESS_GAMMA_RAM);
	}
	else
	{/* dram */
		Mustek_SendData(chip, ES01_A1_HostStartAddr8_15, HIBYTE(RamAccess->LoStartAddress));
		Mustek_SendData(chip, ES01_A2_HostStartAddr16_21, LOBYTE(RamAccess->HiStartAddress) | ACCESS_DRAM);
	}
	
	/* set SDRAM delay time */
	Mustek_SendData(chip, ES01_79_AFEMCLK_SDRAMCLK_DELAY_CONTROL, SDRAMCLK_DELAY_12_ns);
	
	/*Set end address. Unit is a word.*/
	Mustek_SendData(chip, ES01_A3_HostEndAddr0_7, 0xff);
	Mustek_SendData(chip, ES01_A4_HostEndAddr8_15, 0xff);
	Mustek_SendData(chip, ES01_A5_HostEndAddr16_21, 0xff);
	Mustek_ClearFIFO(chip);
	
	if(RamAccess->ReadWrite == WRITE_RAM)
	{/*Write RAM */
		Mustek_DMAWrite(chip,RamAccess->RwSize, RamAccess->BufferPtr); /* read-write size must be even*/

		/*steal read 2byte*/
		usleep(20000);
		RamAccess->RwSize=2;
		RamAccess->BufferPtr = (BYTE*)a;
		RamAccess->ReadWrite = READ_RAM;
		LLFRamAccess(chip,RamAccess);
		DBG (DBG_ASIC,"end steal 2 byte!\n");	
	}
	else
	{/* Read RAM */
		Mustek_DMARead(chip,RamAccess->RwSize, RamAccess->BufferPtr); /* read-write size must be even*/
	}

	DBG (DBG_ASIC,"LLFRamAccess:Exit\n");
	return status;
}


STATUS LLFSetMotorCurrentAndPhase (PAsic chip,LLF_MOTOR_CURRENT_AND_PHASE *MotorCurrentAndPhase)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC,"LLFSetMotorCurrentAndPhase:Enter\n");
	
	BYTE MotorPhase;

	if (MotorCurrentAndPhase->MotorDriverIs3967 == 1)
	{
		MotorPhase = 0xFE;
	}
	else
	{
		MotorPhase = 0xFF;
	}

	DBG(DBG_ASIC,"MotorPhase=0x%x\n",MotorPhase);
	Mustek_SendData(chip, ES02_50_MOTOR_CURRENT_CONTORL, 0x01);

	if (MotorCurrentAndPhase->FillPhase == 0)
	{
		Mustek_SendData(chip, ES01_AB_PWM_CURRENT_CONTROL, 0x00);

		Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, MotorCurrentAndPhase->MotorCurrentTableA[0]);
		Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, MotorCurrentAndPhase->MotorCurrentTableB[0]);
		Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x08 & MotorPhase);

		/*2*/
		Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, MotorCurrentAndPhase->MotorCurrentTableA[0]);
		Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, MotorCurrentAndPhase->MotorCurrentTableB[0]);
		Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x09 & MotorPhase);

		/*3*/
		Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, MotorCurrentAndPhase->MotorCurrentTableA[0]);
		Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, MotorCurrentAndPhase->MotorCurrentTableB[0]);
		Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x01 & MotorPhase);

		/*4*/
		Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, MotorCurrentAndPhase->MotorCurrentTableA[0]);
		Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, MotorCurrentAndPhase->MotorCurrentTableB[0]);
		Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x00 & MotorPhase);
	}
	else
	{
		if (MotorCurrentAndPhase->MoveType == _4_TABLE_SPACE_FOR_FULL_STEP)
		{/* Full Step */
			Mustek_SendData(chip, ES01_AB_PWM_CURRENT_CONTROL, 0x00);

			/*1*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, MotorCurrentAndPhase->MotorCurrentTableA[0]);
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, MotorCurrentAndPhase->MotorCurrentTableB[0]);
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x08 & MotorPhase);

			/*2*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, MotorCurrentAndPhase->MotorCurrentTableA[0]);
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, MotorCurrentAndPhase->MotorCurrentTableB[0]);
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x09 & MotorPhase);

			/*3*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, MotorCurrentAndPhase->MotorCurrentTableA[0]);
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, MotorCurrentAndPhase->MotorCurrentTableB[0]);
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x01 & MotorPhase);

			/*4*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, MotorCurrentAndPhase->MotorCurrentTableA[0]);
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, MotorCurrentAndPhase->MotorCurrentTableB[0]);
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x00 & MotorPhase);
		}
		else if (MotorCurrentAndPhase->MoveType == _8_TABLE_SPACE_FOR_1_DIV_2_STEP)
		{/* Half Step */
			Mustek_SendData(chip, ES01_AB_PWM_CURRENT_CONTROL, 0x01);

			/*1*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, MotorCurrentAndPhase->MotorCurrentTableA[0]);
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, MotorCurrentAndPhase->MotorCurrentTableB[0]);
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x25 & MotorPhase);

			/*2*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, MotorCurrentAndPhase->MotorCurrentTableA[0]);
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, MotorCurrentAndPhase->MotorCurrentTableB[0]);
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x07 & MotorPhase);
	
			/*3*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, MotorCurrentAndPhase->MotorCurrentTableA[0]);
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, MotorCurrentAndPhase->MotorCurrentTableB[0]);
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x24 & MotorPhase);

			/*4*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, MotorCurrentAndPhase->MotorCurrentTableA[0]);
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, MotorCurrentAndPhase->MotorCurrentTableB[0]);
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x30 & MotorPhase);

			/*5*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, MotorCurrentAndPhase->MotorCurrentTableA[0]);
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, MotorCurrentAndPhase->MotorCurrentTableB[0]);
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x2c & MotorPhase);

			/*6*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, MotorCurrentAndPhase->MotorCurrentTableA[0]);
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, MotorCurrentAndPhase->MotorCurrentTableB[0]);
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x0e & MotorPhase);

			/*7*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, MotorCurrentAndPhase->MotorCurrentTableA[0]);
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, MotorCurrentAndPhase->MotorCurrentTableB[0]);
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x2d & MotorPhase);

			/*8*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, MotorCurrentAndPhase->MotorCurrentTableA[0]);
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, MotorCurrentAndPhase->MotorCurrentTableB[0]);
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x39 & MotorPhase);
		}
		else if (MotorCurrentAndPhase->MoveType == _16_TABLE_SPACE_FOR_1_DIV_4_STEP)
		{/* 1/4 step */
			Mustek_SendData(chip, ES01_AB_PWM_CURRENT_CONTROL, 0x02);

			/*1*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(0*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(0*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x08 & MotorPhase);

			/*2*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(1*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(1*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x08 & MotorPhase);

			/*3*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(2*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(2*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x08 & MotorPhase);

			/*4*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(3*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(3*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x08 & MotorPhase);

			/*5*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*cos(0*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*sin(0*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x09 & MotorPhase);

			/*6*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*cos(1*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*sin(1*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x09 & MotorPhase);

			/*7*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*cos(2*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*sin(2*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x09 & MotorPhase);

			/*8*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*cos(3*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*sin(3*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x09 & MotorPhase);

			/*9*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(0*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(0*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x01 & MotorPhase);

			/*10*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(1*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(1*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x01 & MotorPhase);

			/*11*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(2*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(2*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x01 & MotorPhase);

			/*12*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(3*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(3*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x01 & MotorPhase);

			/*13*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*cos(0*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*sin(0*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x00 & MotorPhase);

			/*14*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*cos(1*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*sin(1*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x00 & MotorPhase);

			/*15*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*cos(2*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*sin(2*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x00 & MotorPhase);

			/*16*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*cos(3*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*sin(3*3.141592654*90/4/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x00 & MotorPhase);

		}
		else if (MotorCurrentAndPhase->MoveType == _32_TABLE_SPACE_FOR_1_DIV_8_STEP)
		{
			Mustek_SendData(chip, ES01_AB_PWM_CURRENT_CONTROL, 0x03);

			/*1*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(0*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(0*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x00 & MotorPhase);

			/*2*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(1*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(1*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x00 & MotorPhase);

			/*3*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(2*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(2*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x00 & MotorPhase);

			/*4*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(3*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(3*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x00 & MotorPhase);

			/*5*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(4*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(4*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x00 & MotorPhase);

			/*6*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(5*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(5*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x00 & MotorPhase);

			/*7*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(6*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(6*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x00 & MotorPhase);

			/*8*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(7*3.141592654*90/8/180)) );

			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(7*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x00 & MotorPhase);

			/*9*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(0*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(0*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x08 & MotorPhase);

			/*10*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(1*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(1*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x08 & MotorPhase);

			/*11*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(2*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(2*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x08 & MotorPhase);

			/*12*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(3*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(3*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x08 & MotorPhase);

			/*13*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(4*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(4*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x08 & MotorPhase);

			/*14*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(5*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(5*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x08 & MotorPhase);

			/*15*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(6*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(6*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x08 & MotorPhase);

			/*16*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(7*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(7*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x08 & MotorPhase);

			/*17*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(0*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(0*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x09 & MotorPhase);

			/*18*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(1*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(1*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x09 & MotorPhase);

			/*19*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(2*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(2*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x09 & MotorPhase);

			/*20*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(3*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(3*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x09 & MotorPhase);

			/*21*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(4*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(4*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x09 & MotorPhase);

			/*22*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(5*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(5*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x09 & MotorPhase);

			/*23*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(6*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(6*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x09 & MotorPhase);

			/*24*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(7*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(7*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x09 & MotorPhase);

			/*25*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(0*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(0*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x01 & MotorPhase);

			/*26*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(1*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(1*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x01 & MotorPhase);

			/*27*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(2*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(2*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x01 & MotorPhase);

			/*28*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(3*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(3*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x01 & MotorPhase);

			/*29*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(4*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(4*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x01 & MotorPhase);

			/*30*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(5*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(5*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x01 & MotorPhase);

			/*31*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(6*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(6*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x01 & MotorPhase);

			/*32*/
			Mustek_SendData2Byte(chip, ES02_52_MOTOR_CURRENT_TABLE_A, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableA[0]*sin(7*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_53_MOTOR_CURRENT_TABLE_B, (BYTE)( MotorCurrentAndPhase->MotorCurrentTableB[0]*cos(7*3.141592654*90/8/180)) );
			Mustek_SendData2Byte(chip, ES02_51_MOTOR_PHASE_TABLE_1, 0x01 & MotorPhase);

		}
	}

	if (MotorCurrentAndPhase->FillPhase != 0)
	{
		Mustek_SendData(chip, ES02_50_MOTOR_CURRENT_CONTORL, 0x00 | MotorCurrentAndPhase->MoveType);
	}
	else
	{
		Mustek_SendData(chip, ES02_50_MOTOR_CURRENT_CONTORL, 0x00);
	}

	DBG (DBG_ASIC,"LLFSetMotorCurrentAndPhase:Exit\n");
	return status;
}


STATUS LLFStopMotorMove(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC,"LLFStopMotorMove:Enter\n");
	
	Mustek_SendData(chip, ES01_F4_ActiveTriger, ACTION_TRIGER_DISABLE);
	
	Asic_WaitUnitReady(chip);

	DBG (DBG_ASIC,"LLFStopMotorMove:Exit\n");
	return status;
}

STATUS LLFSetMotorTable (PAsic chip,LLF_SETMOTORTABLE *LLF_SetMotorTable)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC,"LLFSetMotorTable:Enter\n");
	
	LLF_RAMACCESS	RamAccess;
	if (LLF_SetMotorTable->MotorTablePtr != NULL)
	{
		RamAccess.ReadWrite = WRITE_RAM;
		RamAccess.IsOnChipGamma = EXTERNAL_RAM;
		RamAccess.DramDelayTime = SDRAMCLK_DELAY_12_ns;
		
		RamAccess.LoStartAddress = 0;
		RamAccess.LoStartAddress |= LLF_SetMotorTable->MotorTableAddress;
		RamAccess.LoStartAddress <<= TABLE_OFFSET_BASE;
		RamAccess.LoStartAddress |= 0x3000;
		
		RamAccess.HiStartAddress = 0;
		RamAccess.HiStartAddress |= LLF_SetMotorTable->MotorTableAddress;
		RamAccess.HiStartAddress >>= (16 - TABLE_OFFSET_BASE);
		
		RamAccess.RwSize = 512 * 2 * 8; /* BYTE */
		RamAccess.BufferPtr = (BYTE*) LLF_SetMotorTable->MotorTablePtr;
		
		LLFRamAccess(chip,&RamAccess);
		
		/* tell scan chip the motor table address, unit is 2^14 words */
		Mustek_SendData(chip, ES01_9D_MotorTableAddrA14_A21, LLF_SetMotorTable->MotorTableAddress);
	}

	DBG (DBG_ASIC,"LLFSetMotorTable:Exit\n");
	return status;
}

STATUS LLFMotorMove (PAsic chip,LLF_MOTORMOVE *LLF_MotorMove)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC,"LLFMotorMove:Enter\n");
	
	DWORD motor_steps;
	BYTE temp_motor_action;
	BYTE temp_status;

	Mustek_SendData(chip, ES01_F4_ActiveTriger, ACTION_TRIGER_DISABLE);

	status=Asic_WaitUnitReady(chip);

	DBG (DBG_ASIC,"Set start/end pixel\n");

	Mustek_SendData (chip, ES01_B8_ChannelRedExpStartPixelLSB, LOBYTE(100) );
	Mustek_SendData (chip, ES01_B9_ChannelRedExpStartPixelMSB, HIBYTE(100) );
	Mustek_SendData (chip, ES01_BA_ChannelRedExpEndPixelLSB, LOBYTE(101) );
	Mustek_SendData (chip, ES01_BB_ChannelRedExpEndPixelMSB, HIBYTE(101) );

	Mustek_SendData (chip, ES01_BC_ChannelGreenExpStartPixelLSB, LOBYTE(100) );
	Mustek_SendData (chip, ES01_BD_ChannelGreenExpStartPixelMSB, HIBYTE(100) );
	Mustek_SendData (chip, ES01_BE_ChannelGreenExpEndPixelLSB, LOBYTE(101) );
	Mustek_SendData (chip, ES01_BF_ChannelGreenExpEndPixelMSB, HIBYTE(101) );

	Mustek_SendData (chip, ES01_C0_ChannelBlueExpStartPixelLSB, LOBYTE(100) );
	Mustek_SendData (chip, ES01_C1_ChannelBlueExpStartPixelMSB, HIBYTE(100) );
	Mustek_SendData (chip, ES01_C2_ChannelBlueExpEndPixelLSB, LOBYTE(101) );
	Mustek_SendData (chip, ES01_C3_ChannelBlueExpEndPixelMSB, HIBYTE(101) );

	/*set motor accelerate steps MAX 511 steps */
	Mustek_SendData(chip, ES01_E0_MotorAccStep0_7, LOBYTE(LLF_MotorMove->AccStep));
	Mustek_SendData(chip, ES01_E1_MotorAccStep8_8, HIBYTE(LLF_MotorMove->AccStep));
	DBG (DBG_ASIC,"AccStep=%d\n",LLF_MotorMove->AccStep);
	
	Mustek_SendData(chip, ES01_E2_MotorStepOfMaxSpeed0_7, LOBYTE(LLF_MotorMove->FixMoveSteps));
	Mustek_SendData(chip, ES01_E3_MotorStepOfMaxSpeed8_15, HIBYTE(LLF_MotorMove->FixMoveSteps));
	Mustek_SendData(chip, ES01_E4_MotorStepOfMaxSpeed16_19, 0);
	DBG (DBG_ASIC,"FixMoveSteps=%d\n",LLF_MotorMove->FixMoveSteps);
	
	/*set motor decelerate steps MAX 255 steps */
	Mustek_SendData(chip, ES01_E5_MotorDecStep, LLF_MotorMove->DecStep);
	DBG (DBG_ASIC,"DecStep=%d\n",LLF_MotorMove->DecStep);

	/*set motor uniform speed only for uniform speed 
	//only used for UNIFORM_MOTOR_AND_SCAN_SPEED_ENABLE
	//If you use acc mode, this two reg are not used.*/
	Mustek_SendData(chip, ES01_FD_MotorFixedspeedLSB, LOBYTE(LLF_MotorMove->FixMoveSpeed));
	Mustek_SendData(chip, ES01_FE_MotorFixedspeedMSB, HIBYTE(LLF_MotorMove->FixMoveSpeed));
	DBG (DBG_ASIC,"FixMoveSpeed=%d\n",LLF_MotorMove->FixMoveSpeed);
	
	/*Set motor type*/
	Mustek_SendData(chip, ES01_A6_MotorOption, LLF_MotorMove->MotorSelect |
		LLF_MotorMove->HomeSensorSelect | LLF_MotorMove->MotorMoveUnit);

	/*Set motor speed unit, for all motor mode,
	//inclue uniform, acc, motor speed of scan*/
	Mustek_SendData(chip, ES01_F6_MorotControl1, LLF_MotorMove->MotorSpeedUnit | LLF_MotorMove->MotorSyncUnit);

	/* below is setting action register */
	if (LLF_MotorMove->ActionType == ACTION_TYPE_BACKTOHOME)
	{
		DBG (DBG_ASIC,"ACTION_TYPE_BACKTOHOME\n");
		
		temp_motor_action = MOTOR_BACK_HOME_AFTER_SCAN_ENABLE;
		motor_steps = 30000*2;
	}
	else 
	{
		DBG (DBG_ASIC,"Forward or Backward\n");
		temp_motor_action = MOTOR_MOVE_TO_FIRST_LINE_ENABLE;
		motor_steps = LLF_MotorMove->FixMoveSteps ;

		if (LLF_MotorMove->ActionType == ACTION_TYPE_BACKWARD)
		{
			DBG (DBG_ASIC,"ACTION_TYPE_BACKWARD\n");
			temp_motor_action = temp_motor_action | INVERT_MOTOR_DIRECTION_ENABLE;
		}
	}

	if (LLF_MotorMove->ActionType == ACTION_TYPE_TEST_MODE)
	{
		DBG (DBG_ASIC,"ACTION_TYPE_TEST_MODE\n");
		temp_motor_action = temp_motor_action |
							MOTOR_MOVE_TO_FIRST_LINE_ENABLE |
							MOTOR_BACK_HOME_AFTER_SCAN_ENABLE |
							MOTOR_TEST_LOOP_ENABLE;
	}

	Mustek_SendData(chip, ES01_94_PowerSaveControl, 0x27 | LLF_MotorMove->Lamp0PwmFreq | LLF_MotorMove->Lamp1PwmFreq);

	/* fix speed move steps */
	Mustek_SendData(chip, ES01_E2_MotorStepOfMaxSpeed0_7, LOBYTE(motor_steps));
	Mustek_SendData(chip, ES01_E3_MotorStepOfMaxSpeed8_15, HIBYTE(motor_steps));
	Mustek_SendData(chip, ES01_E4_MotorStepOfMaxSpeed16_19, (BYTE)((motor_steps & 0x00ff0000) >> 16));
	DBG (DBG_ASIC,"motor_steps=%d\n",motor_steps);
	DBG (DBG_ASIC,"LOBYTE(motor_steps)=%d\n",LOBYTE(motor_steps));
	DBG (DBG_ASIC,"HIBYTE(motor_steps)=%d\n",HIBYTE(motor_steps));
	DBG (DBG_ASIC,"(BYTE)((motor_steps & 0x00ff0000) >> 16)=%d\n",(BYTE)((motor_steps & 0x00ff0000) >> 16));
	
	if (LLF_MotorMove->ActionMode == ACTION_MODE_UNIFORM_SPEED_MOVE)
	{
		temp_motor_action = temp_motor_action | UNIFORM_MOTOR_AND_SCAN_SPEED_ENABLE;
	}

	Mustek_SendData( chip, ES01_F3_ActionOption, SCAN_DISABLE |
									   SCAN_BACK_TRACKING_DISABLE |
									   temp_motor_action);
	Mustek_SendData(chip, ES01_F4_ActiveTriger, ACTION_TRIGER_ENABLE);

	temp_status = 0;
	if (LLF_MotorMove->WaitOrNoWait == 1)
	{
		if (LLF_MotorMove->ActionType == ACTION_TYPE_BACKTOHOME)
		{
			DBG (DBG_ASIC,"ACTION_TYPE_BACKTOHOME\n");
			Asic_WaitCarriageHome(chip,FALSE);
		}
		else
		{
			Asic_WaitUnitReady(chip);
		}
	}

	DBG (DBG_ASIC,"LLFMotorMove:Exit\n");
	return status;
}

STATUS SetMotorStepTable(PAsic chip,LLF_MOTORMOVE *MotorStepsTable,WORD wStartY, DWORD dwScanImageSteps, WORD wYResolution)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC,"SetMotorStepTable:Enter\n");

	WORD wAccSteps=511;
	WORD wForwardSteps=20;
	BYTE bDecSteps=255;
	WORD wMotorSycnPixelNumber=0;
	WORD wScanAccSteps=511;
	BYTE bScanDecSteps=255;
	WORD wFixScanSteps=20;
	WORD wScanBackTrackingSteps=40;
	WORD wScanRestartSteps=40;
	WORD wScanBackHomeExtSteps=100;
	DWORD dwTotalMotorSteps;

	dwTotalMotorSteps = dwScanImageSteps; 
	
	switch(wYResolution)
	{
	case 2400:
	case 1200:
		wScanAccSteps=100;
		bScanDecSteps=10;
		wScanBackTrackingSteps=10;
		wScanRestartSteps=10;
		break;
	case 600:
	case 300:
		wScanAccSteps=300;
		bScanDecSteps=40;
		break;
	case 150:
		wScanAccSteps=300;
		bScanDecSteps=40;
		break;
	case 100:
	case 75:
	case 50:
		wScanAccSteps=300;
		bScanDecSteps=40;
		break;
	}
	
	if(wStartY<(wAccSteps+wForwardSteps+bDecSteps+wScanAccSteps))  /*not including T0,T1 steps*/
	{	
		wAccSteps=1;
		bDecSteps=1;
		wFixScanSteps=(wStartY-wScanAccSteps)>0 ? 
					  (wStartY-wScanAccSteps):0 ;	
		wForwardSteps=0;
		
		chip->isMotorGoToFirstLine=MOTOR_MOVE_TO_FIRST_LINE_DISABLE;
	}
	else
	{
		wForwardSteps=(wStartY-wAccSteps-(WORD)bDecSteps-wScanAccSteps-wFixScanSteps)>0 ? 
					  (wStartY-wAccSteps-(WORD)bDecSteps-wScanAccSteps-wFixScanSteps):0;

		chip->isMotorGoToFirstLine=MOTOR_MOVE_TO_FIRST_LINE_ENABLE;
	}

	dwTotalMotorSteps+= wAccSteps;
	dwTotalMotorSteps+= wForwardSteps;
	dwTotalMotorSteps+= bDecSteps;
	dwTotalMotorSteps+= wScanAccSteps;
	dwTotalMotorSteps+= wFixScanSteps;
	dwTotalMotorSteps+= bScanDecSteps;
	dwTotalMotorSteps+=2;


	MotorStepsTable->AccStep=wAccSteps;
	MotorStepsTable->DecStep=bDecSteps;
	MotorStepsTable->wForwardSteps=wForwardSteps;
	MotorStepsTable->wScanAccSteps=wScanAccSteps;
	MotorStepsTable->bScanDecSteps=bScanDecSteps;
	MotorStepsTable->wFixScanSteps=wFixScanSteps;
	MotorStepsTable->MotorSyncUnit=(BYTE)wMotorSycnPixelNumber;
	MotorStepsTable->wScanBackHomeExtSteps=wScanBackHomeExtSteps;
	MotorStepsTable->wScanRestartSteps=wScanRestartSteps;
	MotorStepsTable->wScanBackTrackingSteps=wScanBackTrackingSteps;

	/*state 1*/
	Mustek_SendData(chip, ES01_E0_MotorAccStep0_7, LOBYTE(wAccSteps));
	Mustek_SendData(chip, ES01_E1_MotorAccStep8_8, HIBYTE(wAccSteps));
	/*state 2*/
	Mustek_SendData(chip, ES01_E2_MotorStepOfMaxSpeed0_7, LOBYTE(wForwardSteps));
	Mustek_SendData(chip, ES01_E3_MotorStepOfMaxSpeed8_15, HIBYTE(wForwardSteps));
	Mustek_SendData(chip, ES01_E4_MotorStepOfMaxSpeed16_19, 0);
	/*state 3*/
	Mustek_SendData(chip, ES01_E5_MotorDecStep, bDecSteps);
	/*state 4*/
	Mustek_SendData(chip, ES01_AE_MotorSyncPixelNumberM16LSB, LOBYTE(wMotorSycnPixelNumber));
	Mustek_SendData(chip, ES01_AF_MotorSyncPixelNumberM16MSB, HIBYTE(wMotorSycnPixelNumber));
	/*state 5*/
	Mustek_SendData(chip, ES01_EC_ScanAccStep0_7, LOBYTE(wScanAccSteps));
	Mustek_SendData(chip, ES01_ED_ScanAccStep8_8, HIBYTE(wScanAccSteps));
	/*state 6*/
	Mustek_SendData(chip, ES01_EE_FixScanStepLSB, LOBYTE(wFixScanSteps));
	Mustek_SendData(chip, ES01_8A_FixScanStepMSB, HIBYTE(wFixScanSteps));
	/*state 8*/
	Mustek_SendData(chip, ES01_EF_ScanDecStep, bScanDecSteps);
	/*state 10*/
	Mustek_SendData(chip, ES01_E6_ScanBackTrackingStepLSB, LOBYTE(wScanBackTrackingSteps));
	Mustek_SendData(chip, ES01_E7_ScanBackTrackingStepMSB, HIBYTE(wScanBackTrackingSteps));
	/*state 15*/
	Mustek_SendData(chip, ES01_E8_ScanRestartStepLSB, LOBYTE(wScanRestartSteps));
	Mustek_SendData(chip, ES01_E9_ScanRestartStepMSB, HIBYTE(wScanRestartSteps));
	/*state 19*/
	Mustek_SendData(chip, ES01_EA_ScanBackHomeExtStepLSB, LOBYTE(wScanBackHomeExtSteps));
	Mustek_SendData(chip, ES01_EB_ScanBackHomeExtStepMSB, HIBYTE(wScanBackHomeExtSteps));

	/*total motor steps*/
	Mustek_SendData(chip, ES01_F0_ScanImageStep0_7, LOBYTE(dwTotalMotorSteps));
	Mustek_SendData(chip, ES01_F1_ScanImageStep8_15, HIBYTE(dwTotalMotorSteps));
	Mustek_SendData(chip, ES01_F2_ScanImageStep16_19, (BYTE) ((dwTotalMotorSteps & 0x00ff0000) >> 16));

	DBG (DBG_ASIC,"SetMotorStepTable:Exit\n");
	return status;
}

STATUS CalculateMotorTable(LLF_CALCULATEMOTORTABLE *lpCalculateMotorTable, WORD wYResolution)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC,"CalculateMotorTable:Enter\n");
	WORD i;
	WORD wEndSpeed, wStartSpeed;
	WORD wScanAccSteps;
	BYTE bScanDecSteps;
	double PI=3.1415926;
	double x=PI/2;
	long double y;		
	WORD *lpMotorTable;
	

	wStartSpeed=lpCalculateMotorTable->StartSpeed;
	wEndSpeed=lpCalculateMotorTable->EndSpeed;
	wScanAccSteps=lpCalculateMotorTable->AccStepBeforeScan;
	bScanDecSteps=lpCalculateMotorTable->DecStepAfterScan;
	lpMotorTable=lpCalculateMotorTable->lpMotorTable;
	
	/*Motor T0 & T6 Acc Table*/
	for (i = 0; i < 512; i++){
		y=(6000-3500);
		y*=(pow(0.09, (x * i) / 512)-pow(0.09, (x * 511) / 512));
		y+=4500;
		*((LPWORD)lpMotorTable + i			) = (WORD)y;  /*T0*/
		*((LPWORD)lpMotorTable + i + 512 * 6) = (WORD)y;  /*T6*/
	}
	
	/*Motor T1 & T7 Dec Table*/
	for (i = 0; i < 256; i++){
		y=(6000-3500);
		y*=pow(0.3, (x * i) / 256);
		y=6000-y;
		*((LPWORD)lpMotorTable + i + 512	) = (WORD)y;  /*T1*/
		*((LPWORD)lpMotorTable + i + 512 * 7) = (WORD)y;  /*T7*/
	}
	
	switch(wYResolution)
	{
	case 2400:
	case 1200:
	case 600:
	case 300:
	case 150:
	case 100:
	case 75:
	case 50:
		for (i = 0; i < wScanAccSteps; i++)
		{
			y=(wStartSpeed-wEndSpeed);
			y*=(pow(0.09, (x * i) / wScanAccSteps)-pow(0.09, (x * (wScanAccSteps-1)) / wScanAccSteps));
			y+=wEndSpeed;
			*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)y;  /*T2*/
			*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)y;  /*T4*/
		}
		for (i = wScanAccSteps; i < 512; i++)
		{
			*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)wEndSpeed;  /*T2*/
			*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)wEndSpeed;  /*T4*/
		}
		
		
		for (i = 0; i < (WORD)bScanDecSteps; i++)
		{
			y=(wStartSpeed-wEndSpeed);
			y*=pow(0.3, (x * i) / bScanDecSteps);
			y=wStartSpeed-y;
			*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)(y);		/*T3*/
			*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)(y);		/*T5*/
		}
		for (i = bScanDecSteps; i < 256; i++)
		{
			*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)wStartSpeed;  /*T3*/
			*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)wStartSpeed;  /*T5*/
		}
		break;		
	}

	DBG (DBG_ASIC,"CalculateMotorTable:Exit\n");
	return status;
}

STATUS LLFCalculateMotorTable(LLF_CALCULATEMOTORTABLE *LLF_CalculateMotorTable)
{
	DBG (DBG_ASIC,"LLF_CALCULATEMOTORTABLE:Enter\n");
	STATUS status = STATUS_GOOD;
	WORD	i;
	double	PI = 3.1415926535;
	double	x;
	
	x = PI / 2;
	
	for (i = 0; i < 512; i++)
	{
		/* befor scan acc table */
		*(LLF_CalculateMotorTable->lpMotorTable + i			 ) = (WORD)((LLF_CalculateMotorTable->StartSpeed - LLF_CalculateMotorTable->EndSpeed) * pow(0.09, (x * i) / 512) + LLF_CalculateMotorTable->EndSpeed);
		*(LLF_CalculateMotorTable->lpMotorTable + i + 512 * 2) = (WORD)((LLF_CalculateMotorTable->StartSpeed - LLF_CalculateMotorTable->EndSpeed) * pow(0.09, (x * i) / 512) + LLF_CalculateMotorTable->EndSpeed);
		*(LLF_CalculateMotorTable->lpMotorTable + i + 512 * 4) = (WORD)((LLF_CalculateMotorTable->StartSpeed - LLF_CalculateMotorTable->EndSpeed) * pow(0.09, (x * i) / 512) + LLF_CalculateMotorTable->EndSpeed);
		*(LLF_CalculateMotorTable->lpMotorTable + i + 512 * 6) = (WORD)((LLF_CalculateMotorTable->StartSpeed - LLF_CalculateMotorTable->EndSpeed) * pow(0.09, (x * i) / 512) + LLF_CalculateMotorTable->EndSpeed);
	}
	
	for (i = 0; i < 255; i++)
	{
		*(LLF_CalculateMotorTable->lpMotorTable + i + 512 	 ) = (WORD)(LLF_CalculateMotorTable->StartSpeed - (LLF_CalculateMotorTable->StartSpeed - LLF_CalculateMotorTable->EndSpeed) * pow(0.3, (x * i) / 256));
		*(LLF_CalculateMotorTable->lpMotorTable + i + 512 * 3) = (WORD)(LLF_CalculateMotorTable->StartSpeed - (LLF_CalculateMotorTable->StartSpeed - LLF_CalculateMotorTable->EndSpeed) * pow(0.3, (x * i) / 256));
		*(LLF_CalculateMotorTable->lpMotorTable + i + 512 * 5) = (WORD)(LLF_CalculateMotorTable->StartSpeed - (LLF_CalculateMotorTable->StartSpeed - LLF_CalculateMotorTable->EndSpeed) * pow(0.3, (x * i) / 256));
		*(LLF_CalculateMotorTable->lpMotorTable + i + 512 * 7) = (WORD)(LLF_CalculateMotorTable->StartSpeed - (LLF_CalculateMotorTable->StartSpeed - LLF_CalculateMotorTable->EndSpeed) * pow(0.3, (x * i) / 256));
	}
	
	for (i = 0; i < 512; i++)
	{/* back acc table */
		*(LLF_CalculateMotorTable->lpMotorTable + i			 ) = (WORD)((LLF_CalculateMotorTable->StartSpeed - LLF_CalculateMotorTable->EndSpeed) * pow(0.09, (x * i) / 512) + LLF_CalculateMotorTable->EndSpeed);
		*(LLF_CalculateMotorTable->lpMotorTable + i + 512 * 6) = (WORD)((LLF_CalculateMotorTable->StartSpeed - LLF_CalculateMotorTable->EndSpeed) * pow(0.09, (x * i) / 512) + LLF_CalculateMotorTable->EndSpeed);
	}

	if (LLF_CalculateMotorTable->AccStepBeforeScan == 0)
	{
	}
	else
	{
		for (i = 0; i < LLF_CalculateMotorTable->AccStepBeforeScan; i++)
		{
			*(LLF_CalculateMotorTable->lpMotorTable + i + 512 * 2) = 
				(WORD)((LLF_CalculateMotorTable->StartSpeed - LLF_CalculateMotorTable->EndSpeed) * 
				(pow(0.09, (x * i) / LLF_CalculateMotorTable->AccStepBeforeScan) -pow(0.09, (x * (LLF_CalculateMotorTable->AccStepBeforeScan-1)) / LLF_CalculateMotorTable->AccStepBeforeScan))+ LLF_CalculateMotorTable->EndSpeed);
		}
	}

	DBG (DBG_ASIC,"LLF_CALCULATEMOTORTABLE:Exit\n");
	return status;
}


STATUS SetMotorCurrent(PAsic chip,WORD dwMotorSpeed, LLF_MOTOR_CURRENT_AND_PHASE *CurrentPhase)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC,"SetMotorCurrent:Enter\n");
	
	if(dwMotorSpeed<2000)
	{
		CurrentPhase->MotorCurrentTableA[0]=255;
		CurrentPhase->MotorCurrentTableB[0]=255;
	}
	else if(dwMotorSpeed<3500)
	{
		CurrentPhase->MotorCurrentTableA[0]=200;
		CurrentPhase->MotorCurrentTableB[0]=200;
	}
	else if(dwMotorSpeed<5000)
	{
		CurrentPhase->MotorCurrentTableA[0]=160;
		CurrentPhase->MotorCurrentTableB[0]=160;
	}
	else if(dwMotorSpeed<10000)
	{
		CurrentPhase->MotorCurrentTableA[0]=70;
		CurrentPhase->MotorCurrentTableB[0]=70;
	}
	else if(dwMotorSpeed<17000)
	{
		CurrentPhase->MotorCurrentTableA[0]=60;
		CurrentPhase->MotorCurrentTableB[0]=60;
	}
	else if(dwMotorSpeed<25000)
	{
		CurrentPhase->MotorCurrentTableA[0]=50;
		CurrentPhase->MotorCurrentTableB[0]=50;
	}
	else
	{
		CurrentPhase->MotorCurrentTableA[0]=50;
		CurrentPhase->MotorCurrentTableB[0]=50;
	}

	DBG (DBG_ASIC,"SetMotorCurrent:Exit\n");
	return status;
}


STATUS MotorBackHome (PAsic chip,BYTE WaitOrNoWait)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC,"MotorBackHome:Enter\n");
	WORD BackHomeMotorTable[512*8];
	
	LLF_CALCULATEMOTORTABLE CalMotorTable;
	CalMotorTable.StartSpeed = 5000; 
	CalMotorTable.EndSpeed =1200;
	CalMotorTable.AccStepBeforeScan = 511;
	CalMotorTable.DecStepAfterScan = 255;
	CalMotorTable.lpMotorTable = BackHomeMotorTable;
	LLFCalculateMotorTable(&CalMotorTable);
	
	
	LLF_MOTOR_CURRENT_AND_PHASE CurrentPhase;
	CurrentPhase.MotorCurrentTableA[0] = 220;
	CurrentPhase.MotorCurrentTableB[0] = 220;
	CurrentPhase.MoveType = _4_TABLE_SPACE_FOR_FULL_STEP;
	LLFSetMotorCurrentAndPhase (chip,&CurrentPhase);
	
	LLF_SETMOTORTABLE LLF_SetMotorTable;
	LLF_SetMotorTable.MotorTableAddress = 0;
	LLF_SetMotorTable.MotorTablePtr = BackHomeMotorTable;
	LLFSetMotorTable (chip,&LLF_SetMotorTable);
	
	LLF_MOTORMOVE MotorMove;
	MotorMove.MotorSelect = MOTOR_0_ENABLE | MOTOR_1_DISABLE;
	MotorMove.MotorMoveUnit = ES03_TABLE_DEFINE;
	MotorMove.MotorSpeedUnit = SPEED_UNIT_1_PIXEL_TIME;
	MotorMove.MotorSyncUnit = MOTOR_SYNC_UNIT_1_PIXEL_TIME;
	MotorMove.HomeSensorSelect = HOME_SENSOR_0_ENABLE;
	MotorMove.ActionMode = ACTION_MODE_ACCDEC_MOVE;
	MotorMove.ActionType = ACTION_TYPE_BACKTOHOME;
	
	MotorMove.AccStep = 511;
	MotorMove.DecStep = 255;
	MotorMove.FixMoveSteps = 0;
	MotorMove.FixMoveSpeed = 3000;
	MotorMove.WaitOrNoWait = WaitOrNoWait;
	LLFMotorMove (chip,&MotorMove);

	DBG (DBG_ASIC,"MotorBackHome:Exit\n");
	return status;
}


STATUS LLFSetRamAddress(PAsic chip,DWORD dwStartAddr, DWORD dwEndAddr, BYTE byAccessTarget)
{
	STATUS status = STATUS_GOOD;
	DBG (DBG_ASIC,"LLFSetRamAddress:Enter\n");
	
	LPBYTE	pStartAddr	= (LPBYTE) &dwStartAddr;
	LPBYTE	pEndAddr	= (LPBYTE) &dwEndAddr;
	
	/*Set start address. */
	Mustek_SendData(chip,ES01_A0_HostStartAddr0_7,	*(pStartAddr)   );
	Mustek_SendData(chip,ES01_A1_HostStartAddr8_15,	*(pStartAddr+1) );
	if(byAccessTarget == ACCESS_DRAM)
		Mustek_SendData(chip,ES01_A2_HostStartAddr16_21, *(pStartAddr+2) | ACCESS_DRAM );
	else
		Mustek_SendData(chip,ES01_A2_HostStartAddr16_21, *(pStartAddr+2) | ACCESS_GAMMA_RAM );

	/*Set end address. */
	Mustek_SendData(chip,ES01_A3_HostEndAddr0_7, *(pEndAddr)   );
	Mustek_SendData(chip,ES01_A4_HostEndAddr8_15, *(pEndAddr+1) );
	Mustek_SendData(chip,ES01_A5_HostEndAddr16_21,	*(pEndAddr+2) );
	
	Mustek_ClearFIFO(chip);

	DBG (DBG_ASIC,"LLFSetRamAddress:Exit\n");
	return status;
}

